@@ -461,6 +461,19 @@ class pReMiuMHyperParams{
 		void rateTauCAR(const double& b){
 			_rateTauCAR = b;
 		}
+
+		vector<double> initAlloc() const{
+			return _initAlloc;
+		}
+
+		void initAlloc(const vector<double>& c){
+			_initAlloc = c;
+		}
+
+		double initAlloc(const unsigned int& j) const{
+			return _initAlloc[j];
+		}
+
 		// Copy operator
 		pReMiuMHyperParams& operator=(const pReMiuMHyperParams& hyperParams){
 			_shapeAlpha = hyperParams.shapeAlpha();
@@ -493,6 +506,7 @@ class pReMiuMHyperParams{
 			_truncationEps = hyperParams.truncationEps();
 			_shapeTauCAR = hyperParams.shapeTauCAR();
 			_rateTauCAR = hyperParams.rateTauCAR();
+			_initAlloc = hyperParams.initAlloc();
 			return *this;
 		}
 
@@ -571,6 +585,9 @@ class pReMiuMHyperParams{
 		double _shapeTauCAR;
 		double _rateTauCAR;
 
+		// Initial allocations 
+		vector<double> _initAlloc;
+
 };
 
 
@@ -595,12 +612,13 @@ class pReMiuMParams{
 				const unsigned int& nPredictSubjects,
 				const vector<unsigned int>& nCategories,
 				const unsigned int& nClusInit,
-				const string covariateType){
+				const string covariateType,
+				const bool weibullFixedShape){
 
 			unsigned int nDiscrCovs = 0;
 
 			// Initially make the maximum number of clusters  the bigger or
-			// the initial number of clusters and 100.
+			// the initial number of clusters and 150.
 			// This is only used for initial memory allocation
 			// And will ensure that at this initial time sufficient
 			// space is allocated to prevent future allocations
@@ -721,6 +739,11 @@ class pReMiuMParams{
 				}
 			}
 			_uCAR.resize(nSubjects);
+			if (weibullFixedShape) {
+				_nu.resize(1);
+			} else {
+				_nu.resize(maxNClusters);
+			}
 		}
 
 		/// \brief Return the number of clusters
@@ -747,6 +770,7 @@ class pReMiuMParams{
 				_logPsi.resize(nClus);
 				_v.resize(nClus);
 				_theta.resize(nClus);
+				if (_nu.size()>1) _nu.resize(nClus);
 				for (unsigned int c=0;c<nClus;c++){
 					_theta[c].resize(nCategoriesY);
 				}
@@ -1557,16 +1581,27 @@ class pReMiuMParams{
 			_sigmaSqY=sigmaSqYVal;
 		}
 
-		/// \brief Return the parameter nu
-		double nu() const{
+		/// \brief Return the vector _nu
+		vector<double> nu() const{
 			return _nu;
 		}
 
-		/// \brief Set the parameter nu
-		void nu(const double& nuVal){
+		/// \brief Return the parameter _nu for cluster c
+		double nu(const unsigned int& c ) const{
+			return _nu[c];
+		}
+
+		/// \brief Set the vector nu
+		void nu(vector<double>& nuVal){
 			_nu=nuVal;
 		}
 
+		/// \brief Set the parameter nu for cluster c
+		void nu(const unsigned int& c, double& nuVal){
+			_nu[c]=nuVal;
+		}
+
+
 		/// \brief Return the vector _uCAR
 		vector<double> uCAR() const{
 			return _uCAR;
@@ -1972,7 +2007,7 @@ class pReMiuMParams{
 		double _sigmaSqY;
 
 		/// \brief Prior shape parameter for Y model when response is weibull (survival)
-		double _nu;
+		vector<double> _nu;
 
 		/// NOTE: hyper parameters
 		/// \brief The hyper parameter object
@@ -2132,6 +2167,20 @@ double logPYiGivenZiWiNormal(const pReMiuMParams& params, const pReMiuMData& dat
 	return logPdfNormal(dataset.continuousY(i),mu,sqrt(params.sigmaSqY()));
 }
 
+double logPYiGivenZiWiNormalSpatial(const pReMiuMParams& params, const pReMiuMData& dataset,
+						const unsigned int& nFixedEffects,const int& zi,
+						const unsigned int& i){
+
+	double mu;
+	mu=params.theta(zi,0);
+	for(unsigned int j=0;j<nFixedEffects;j++){
+		mu+=params.beta(j,0)*dataset.W(i,j);
+	}
+	mu+=params.uCAR(i);
+
+	return logPdfNormal(dataset.continuousY(i),mu,sqrt(params.sigmaSqY()));
+}
+
 double logPYiGivenZiWiCategorical(const pReMiuMParams& params, const pReMiuMData& dataset,
 						const unsigned int& nFixedEffects,
 						const int& zi,
@@ -2165,11 +2214,21 @@ double logPYiGivenZiWiSurvival(const pReMiuMParams& params, const pReMiuMData& d
 						const unsigned int& nFixedEffects,const int& zi,
 						const unsigned int& i){
 
+	unsigned int weibullFixedShape=params.nu().size();
+
 	double lambda = params.theta(zi,0);
 	for(unsigned int j=0;j<nFixedEffects;j++){
 		lambda+=params.beta(j,0)*dataset.W(i,j);
 	}
-	return logPdfWeibullCensored(dataset.continuousY(i), params.nu(), exp(lambda), dataset.censoring(i));
+	double nu=0;
+	
+	if (weibullFixedShape==1) {
+		nu=params.nu(0);
+	} else {
+		nu=params.nu(zi);
+	}
+
+	return logPdfWeibullCensored(dataset.continuousY(i), nu, exp(lambda), dataset.censoring(i));
 }
 
 
@@ -2195,6 +2254,7 @@ vector<double> pReMiuMLogPost(const pReMiuMParams& params,
 	vector<unsigned int> nCategories = dataset.nCategories();
 	const pReMiuMHyperParams& hyperParams = params.hyperParams();
 	const bool includeCAR=model.options().includeCAR();
+	const bool weibullFixedShape=model.options().weibullFixedShape();
 
 	// (Augmented) Log Likelihood first
 	// We want p(y,X|z,params,W) = p(y|z=c,W,params)p(X|z=c,params)
@@ -2265,7 +2325,11 @@ vector<double> pReMiuMLogPost(const pReMiuMParams& params,
 		}else if(outcomeType.compare("Categorical")==0){
 			logPYiGivenZiWi = &logPYiGivenZiWiCategorical;
 		}else if(outcomeType.compare("Normal")==0){
-			logPYiGivenZiWi = &logPYiGivenZiWiNormal;
+				if (includeCAR){
+					logPYiGivenZiWi = &logPYiGivenZiWiNormalSpatial;
+				}else{
+					logPYiGivenZiWi = &logPYiGivenZiWiNormal;
+				}
 		}else if(outcomeType.compare("Survival")==0){
 			logPYiGivenZiWi = &logPYiGivenZiWiSurvival;
 		}
@@ -2410,7 +2474,13 @@ vector<double> pReMiuMLogPost(const pReMiuMParams& params,
 		}
 
 		if(outcomeType.compare("Survival")==0){
-			logPrior+=logPdfGamma(params.nu(),hyperParams.shapeNu(),hyperParams.scaleNu());
+			if (weibullFixedShape){
+				logPrior+=logPdfGamma(params.nu(0),hyperParams.shapeNu(),hyperParams.scaleNu());
+			} else {
+				for (unsigned int c=0;c<maxNClusters;c++) {
+					logPrior+=logPdfGamma(params.nu(c),hyperParams.shapeNu(),hyperParams.scaleNu());
+				}
+			}
 		}
 
 		// Prior for TauCAR and UCAR
@@ -2591,7 +2661,11 @@ double logCondPostThetaBeta(const pReMiuMParams& params,
 	}else if(outcomeType.compare("Categorical")==0){
 		logPYiGivenZiWi = &logPYiGivenZiWiCategorical;
 	}else if(outcomeType.compare("Normal")==0){
-		logPYiGivenZiWi = &logPYiGivenZiWiNormal;
+		if (includeCAR){
+			logPYiGivenZiWi = &logPYiGivenZiWiNormalSpatial;
+		}else{
+			logPYiGivenZiWi = &logPYiGivenZiWiNormal;
+		}
 	}else if(outcomeType.compare("Survival")==0){
 		logPYiGivenZiWi = &logPYiGivenZiWiSurvival;
 	}
@@ -2734,7 +2808,7 @@ void logUiPostPoissonSpatial(const pReMiuMParams& params,
 // given \gamma for adaptive rejection sampling
 void logNuPostSurvival(const pReMiuMParams& params,
                                  const mcmcModel<pReMiuMParams,pReMiuMOptions,pReMiuMData>& model,
-                                 const unsigned int& iSub,
+                                 const unsigned int& cluster,
                                  const double& x,
                                  double* Pt_y1, double* Pt_y2){
 
@@ -2744,29 +2818,55 @@ void logNuPostSurvival(const pReMiuMParams& params,
 	unsigned int nSubjects=dataset.nSubjects();
 	const vector<unsigned int> censoring = dataset.censoring();
 	vector<double> y = dataset.continuousY();
+	const bool weibullFixedShape=model.options().weibullFixedShape();
+
 	double y1;
 	double y2;
  	// compute d
 	double dCensored = 0;
-	for (unsigned int i=0;i<nSubjects;i++){
-		dCensored += censoring[i];
-	}	
+	
+	if (weibullFixedShape){
+		for (unsigned int i=0;i<nSubjects;i++){
+			dCensored += censoring[i];
+		}	
+	} else {
+		for (unsigned int i=0;i<nSubjects;i++){
+			int zi=params.z(i);
+			if (zi==(int)cluster) dCensored += censoring[i];
+		}	
+	}
 	// sum of yi^nu
 	double yNu = 0;
 	double yNulogy = 0;
 	for (unsigned int i=0;i<nSubjects;i++){
 		int zi=params.z(i);
-		double lambda = params.theta(zi,0);
-		for(unsigned int j=0;j<nFixedEffects;j++){
-			lambda+=params.beta(j,0)*dataset.W(i,j);
+		if (weibullFixedShape){
+			double lambda = params.theta(zi,0);
+			for(unsigned int j=0;j<nFixedEffects;j++){
+				lambda+=params.beta(j,0)*dataset.W(i,j);
+			}
+			yNulogy += pow(y[i],x) * log(y[i]) * exp(lambda);
+			yNu += pow(y[i],x) * exp(lambda);
+		} else {
+			if (zi==(int)cluster) {
+				double lambda = params.theta(zi,0);
+				for(unsigned int j=0;j<nFixedEffects;j++){
+					lambda+=params.beta(j,0)*dataset.W(i,j);
+				}
+				yNulogy += pow(y[i],x) * log(y[i]) * exp(lambda);
+				yNu += pow(y[i],x) * exp(lambda);
+			}
 		}
-		yNulogy += pow(y[i],x) * log(y[i]) * exp(lambda);
-		yNu += pow(y[i],x) * exp(lambda);
 	}	
 	// sum of di*log yi
 	double dlogY = 0;
 	for (unsigned int i=0;i<nSubjects;i++){
-		dlogY += log(y[i]) * censoring[i];
+		if (weibullFixedShape){
+			dlogY += log(y[i]) * censoring[i];
+		} else {
+			int zi=params.z(i);
+			if (zi==(int)cluster) dlogY += log(y[i]) * censoring[i];
+		}
 	}	
 	y1=dCensored * log(x) - yNu + x * dlogY + (hyperParams.shapeNu()-1) * log(x) - hyperParams.scaleNu() * x;
 	// derivative of y1
